// # Globbing
// for performance reasons we're only matching one level down:
// 'test/spec/{,*/}*.js'
// use this if you want to match all subfolders:
// 'test/spec/**/*.js'

module.exports = function(grunt) {

  // configurable paths
  var ohcmsConfig = {
      app: 'public_html/ohcms',
      dist: 'dist/public_html',
      theme: 'default',
  };

  grunt.initConfig({
    ohcms: ohcmsConfig,
    pkg: grunt.file.readJSON('package.json'),
    watch: {
      less: {
        files: ['<%= ohcms.app %>/styles/less/{,*/}*.less'],
        tasks: ['less:ohcms']
      },
      website: {
        files: ['styles/scss/{,*/}*.scss'],
        tasks: ['sass:website']
      }
    },
    jshint: {
      all: ['Gruntfile.js', '<%= ohcms.app %>/scripts/**/*.js', '!<%= ohcms.app %>/scripts/vendor/**/*.js']
    },
    mocha: {
      unit: {
        src: ['<%= ohcms.app %>/test/unit/index.html'],
        options: {
          reporter: 'Spec',
          run: false
        }
      },
      e2e: {
        src: ['<%= ohcms.app %>/test/e2e/index.html'],
        options: {
          reporter: 'Spec',
          run: false
        }
      }
    },
    clean: {
      dist: ['.tmp', 'dist/Astro/*', 'dist/Comet/*', 'dist/vendor/*', 'dist/public_html/ohcms/*'],
      server: '.tmp'
    },
    less: {
      ohcms: {
        options: {
          compress: true
        },
        files: {
          '<%= ohcms.app %>/styles/css/bootstrap.css': '<%= ohcms.app %>/styles/themes/<%= ohcms.theme %>/bootstrap.less',
          '<%= ohcms.app %>/styles/css/adminflare.css': '<%= ohcms.app %>/styles/themes/<%= ohcms.theme %>/adminflare.less',
          '<%= ohcms.app %>/styles/css/pages.css': '<%= ohcms.app %>/styles/themes/pages.less',
          '<%= ohcms.app %>/styles/css/ohcms.css': '<%= ohcms.app %>/styles/less/ohcms/ohcms.less'
        }
      }
    },
    sass: {
      website: {
        options: {
          sourcemap: 'none',
          style: 'compressed'
        },
        files: {
          'styles/css/screen.css': 'styles/scss/screen.scss'
        }
      }
    },
    // not used since Uglify task does concat,
    // but still available if needed
    /*concat: {
      dist: {}
    },*/
    requirejs: {
      dist: {
        // Options: https://github.com/jrburke/r.js/blob/master/build/example.build.js
        options: {
          // `name` and `out` is set by grunt-usemin
          baseUrl: '<%= ohcms.app %>/scripts',
          optimize: 'none',
          preserveLicenseComments: false,
          useStrict: true,
          wrap: true
          //uglify2: {} // https://github.com/mishoo/UglifyJS2
        }
      }
    },
    useminPrepare: {
      html: '<%= ohcms.app %>/index.html',
      options: {
        dest: '<%= ohcms.dist %>'
      }
    },
    usemin: {
      html: ['<%= ohcms.dist %>/{,*/}*.html'],
      css: ['<%= ohcms.dist %>/ohcms/styles/{,*/}*.css'],
      options: {
      }
    },
    htmlmin: {
      dist: {
        options: {
          /*removeCommentsFromCDATA: true,
          // https://github.com/yeoman/grunt-usemin/issues/44
          //collapseWhitespace: true,
          collapseBooleanAttributes: true,
          removeAttributeQuotes: true,
          removeRedundantAttributes: true,
          useShortDoctype: true,
          removeEmptyAttributes: true,
          removeOptionalTags: true*/
          removeComments: false
          },
          files: [{
            expand: true,
            cwd: '<%= ohcms.app %>',
            src: '*.html',
            dest: '<%= ohcms.dist %>/ohcms'
        }]
      }
    },
    copy: {
      dist: {
        files: [
          {src: ['Astro/**', 'Comet/**', 'vendor/**'], dest: 'dist/', filter: 'isFile'},
          {src: ['public_html/index.php', 'public_html/.htaccess', '**'], dest: 'dist/', filter: 'isFile'},
          {src: ['<%= ohcms.app %>/api/**', '<%= ohcms.app %>/api/.htaccess'], dest: 'dist/', filter: 'isFile'},
          {src: ['<%= ohcms.app %>/fonts/**'], dest: 'dist/', filter: 'isFile'},
          {src: ['<%= ohcms.app %>/img/**'], dest: 'dist/', filter: 'isFile'},
          {src: ['<%= ohcms.app %>/templates/**'], dest: 'dist/', filter: 'isFile'},
          {src: ['<%= ohcms.app %>/components/font-awesome/css/font-awesome-ie7.min.css'], dest: 'dist/'},
          {src: ['<%= ohcms.app %>/components/flot/excanvas.min.js'], dest: 'dist/'},
          {src: ['<%= ohcms.app %>/.htaccess'], dest: 'dist/'}
        ]
      }
    }
  });

  // Load plugins
  grunt.loadNpmTasks('grunt-contrib-watch');
  grunt.loadNpmTasks('grunt-contrib-jshint');
  grunt.loadNpmTasks('grunt-contrib-less');
  grunt.loadNpmTasks('grunt-contrib-sass');
  grunt.loadNpmTasks('grunt-contrib-concat');
  grunt.loadNpmTasks('grunt-contrib-uglify');
  grunt.loadNpmTasks('grunt-contrib-htmlmin');
  grunt.loadNpmTasks('grunt-contrib-clean');
  grunt.loadNpmTasks('grunt-contrib-copy');
  grunt.loadNpmTasks('grunt-contrib-requirejs');
  grunt.loadNpmTasks('grunt-usemin');
  grunt.loadNpmTasks('grunt-mocha');

  // Register Tasks
  grunt.registerTask('test', function() {
    var themes = ['black-blue', 'black-cyan', 'black-green', 'black-orange', 'black-pink', 'black-red', 'cyan', 'default', 'green', 'orange', 'red'];
    
    grunt.log.writeln('Testing compilation of themes ("' + themes.join('", "') + '") and sass!');
    
    for (var i in themes) {
      ohcmsConfig.theme = themes[i];
      grunt.config('ohcms', ohcmsConfig);
      grunt.task.run(['less']);
    }
    grunt.task.run(['sass']);
  });

  grunt.registerTask('build', [
    'clean:dist',
    'less',
    'sass',
    'useminPrepare',
    'requirejs',
    'htmlmin',
    'concat',
    'uglify',
    'copy',
    'usemin'
  ]);

  // leaving this here in case
  // removing it will break something
  grunt.registerTask('lessmin:website', [
  ]);

  grunt.registerTask('default', [
    'jshint',
    'less',
    'sass'
  ]);

};
