/*!
 * ohslideshow jQuery plugin
 * author: Isaac Wong
 */

// the semi-colon before the function invocation is a safety
// net against concatenated scripts and/or other plugins
// that are not closed properly.
;(function ( $, window, document, undefined ) {

  // undefined is used here as the undefined global
  // variable in ECMAScript 3 and is mutable (i.e. it can
  // be changed by someone else). undefined isn't really
  // being passed in so we can ensure that its value is
  // truly undefined. In ES5, undefined can no longer be
  // modified.

  // window and document are passed through as local
  // variables rather than as globals, because this (slightly)
  // quickens the resolution process and can be more
  // efficiently minified (especially when both are
  // regularly referenced in your plugin).

  // Create the defaults once
  var pluginName = "ohslideshow",
    defaults = {
      slideIntervalId: null,
      slideDuration: 6000,
      slideSpeed: 600,
      slideTransitionEasing: 'easeOutCirc',
      showArrowNav: true,
      showDotNav: true,
      startOnLoad: false,
      auto: false,
      responsive: true,
      maxWidth: 700,
      customPrevBtn: null,
      customNextBtn: null
    };

  // The actual plugin constructor
  function Plugin( element, options ) {
    this.element = element;

    // jQuery has an extend method that merges the
    // contents of two or more objects, storing the
    // result in the first object. The first object
    // is generally empty because we don't want to alter
    // the default options for future instances of the plugin
    this.options = $.extend( {}, defaults, options);

    this._defaults = defaults;
    this._name = pluginName;
    
    return this;
  }

  Plugin.prototype = {

    init: function() {
      // Place initialization logic here
      // You already have access to the DOM element and
      // the options via the instance, e.g. this.element
      // and this.options
      // you can add more functions like the one below and
      // call them like so: this.yourOtherFunction(this.element, this.options).
      var that;

      that = this;

      this.assignDataIndexToEachSlide(this.element, this.options);
      this.adjustSlideshowLength(this.element, this.options);
      $(window).resize($.proxy(this.adjustSlideshowLength, this));

      if (this.options.showArrowNav) {
        this.addArrowNav(this.element, this.options);
      }

      if (this.options.showDotNav) {
        this.addDotNav(this.element, this.options);
      }

      if (this.options.startOnLoad) {
        this.start(this.element, this.options);
      }

      if (this.options.responsive) {
        this.adjustResponsive(this.element, this.options);
        $(window).resize(function() {
          that.adjustResponsive(that.element, that.options);
        });
      }
    },

    addArrowNav: function(el, options) {
      var that, prevBtn, nextBtn;

      that = this;

      if (options.customPrevBtn && $(options.customPrevBtn).length) {
        prevBtn = $(options.customPrevBtn);
      } else {
        prevBtn = $('<a href="javascript:void(0);" class="slideshow-btn-prev"></a>').appendTo(el);
      }

      if (options.customNextBtn && $(options.customNextBtn).length) {
        nextBtn = $(options.customNextBtn);
      } else {
        nextBtn = $('<a href="javascript:void(0);" class="slideshow-btn-next"></a>').appendTo(el);
      }

      prevBtn.on('click', function() {
        that.prev(el, options);
      });

      nextBtn.on('click', function() {
        that.next(el, options);
      });
    },

    addDotNav: function(el, options) {
      var that, numberOfSlides, i, dotNavHtml;

      that = this;
      numberOfSlides = $(el).find('ul.slideshow li').length;
      dotNavHtml = '';

      $('<ul class="slideshow-dot-nav"></ul>').appendTo(el);

      for (i = 0; i < numberOfSlides; i++) {
        dotNavHtml += '<li><a href="javascript:void(0);" class="slideshow-dot fa fa-circle-o"></a></li>';
      }

      $(el).find('.slideshow-dot-nav').append(dotNavHtml);
      this.setActiveDot(el, options, 0);

      $(el).find('.slideshow-dot-nav li').click(function() {
        var currentSlideIndex, newSlideIndex, numberOfSlidesToShift;

        // prevent button jamming
        if ($(el).find('ul.slideshow').queue("fx") != 0) return false;

        currentSlideIndex = $(el).find('.slideshow-dot-nav li a.fa-circle').parent('li').index();
        newSlideIndex = $(this).index();

        if (newSlideIndex === currentSlideIndex) {
          return;
        } else if (newSlideIndex > currentSlideIndex) {
          that.gotoSlideShiftLeft(el, options, newSlideIndex);
        } else {
          numberOfSlidesToShift = currentSlideIndex - newSlideIndex;
          that.gotoSlideShiftRight(el, options, newSlideIndex, numberOfSlidesToShift);
        }
      });
    },

    assignDataIndexToEachSlide: function(el, options) {
      $(el).find('ul.slideshow li').each(function(index){
        $(this).attr('data-index', index);
      });
    },

    adjustSlideshowLength: function(el, options) {
      // wide container holding all slides
      var imageOuterWidth, numberOfSlides, adjustmentPadding, finalWidth;

      imageOuterWidth = $(this.element).find('ul.slideshow li img').outerWidth();
      numberOfSlides = $(this.element).find('ul.slideshow li').length;
      adjustmentPadding = 100;
      finalWidth = imageOuterWidth * numberOfSlides + adjustmentPadding;

      $(this.element).find('ul.slideshow').width(finalWidth);
    },

    adjustResponsive: function(el, options) {
      var followWidth, newWidth;

      followWidth = $(el).parent('div').width();
      newWidth = followWidth > options.maxWidth ? options.maxWidth : followWidth;

      $(el).width(newWidth);
      $(el).find('ul.slideshow li img').width(newWidth);
    },

    start: function(el, options) {
      var that;

      that = this;

      clearInterval(this.options.slideIntervalId);

      options.slideIntervalId = setInterval(function() {
        that.next(el, options);
      }, options.slideDuration);
    },

    pause: function(el, options) {
      clearInterval(options.slideIntervalId);
    },

    prev: function(el, options) {
      var slideWidth;

      // prevent button jamming
      if ($(el).find('ul.slideshow').queue("fx") != 0) return false;

      this.pause(el, options);

      slideWidth = $(el).find('ul.slideshow li').outerWidth();

      // move last <li> to front
      $(el).find('ul.slideshow li:first').before($(el).find('ul.slideshow li:last'));
      $(el).find('ul.slideshow').css('margin-left', '-=' + slideWidth);
      this.setActiveDot(el, options, $(el).find('ul.slideshow li:first').data('index'));

      $(el).find('ul.slideshow').animate({
        'margin-left': '+=' + slideWidth
      }, options.slideSpeed, options.slideTransitionEasing);

      if (options.auto) {
        this.start(el, options);
      }
    },

    next: function(el, options) {
      var slideWidth;

      // prevent button jamming
      if ($(el).find('ul.slideshow').queue("fx") != 0) return false;

      this.pause(el, options);

      slideWidth = $(el).find('ul.slideshow li').outerWidth();

      this.setActiveDot(el, options, $(el).find('ul.slideshow li:nth-child(2)').data('index'));

      $(el).find('ul.slideshow').animate({
        'margin-left': '-=' + slideWidth
      }, options.slideSpeed, options.slideTransitionEasing, function() {
        // move first <li> to end
        $(el).find('ul.slideshow li:last').after($(el).find('ul.slideshow li:first'));
        $(el).find('ul.slideshow').css('margin-left', '+=' + slideWidth);
      });

      if (options.auto) {
        this.start(el, options);
      }
    },

    gotoSlideShiftLeft: function(el, options, slideIndex) {
      var slideWidth, slideIndexCurrentPosition;

      this.setActiveDot(el, options, slideIndex);

      // prevent button jamming
      if ($(el).find('ul.slideshow').queue("fx") != 0) return false;

      this.pause(el, options);

      slideIndexCurrentPosition = $(el).find('ul.slideshow li[data-index="'+slideIndex+'"]').index();
      slideWidth = $(el).find('ul.slideshow li').outerWidth();

      for (var i = 0; i < slideIndexCurrentPosition; i++) {
        $(el).find('ul.slideshow').animate({
          'margin-left': '-=' + slideWidth
        }, (i < slideIndexCurrentPosition - 1) ? 100 : options.slideSpeed, options.slideTransitionEasing, function() {
          $(el).find('ul.slideshow li:last').after($(el).find('ul.slideshow li:first'));
          $(el).find('ul.slideshow').css('margin-left', '+=' + slideWidth);
        });
      }

      if (options.auto) {
        this.start(el, options);
      }
    },

    gotoSlideShiftRight: function(el, options, slideIndex, numberOfSlidesToShift) {
      var slideWidth, slideIndexCurrentPosition;

      this.setActiveDot(el, options, slideIndex);

      // prevent button jamming
      if ($(el).find('ul.slideshow').queue("fx") != 0) return false;

      this.pause(el, options);

      slideWidth = $(el).find('ul.slideshow li').outerWidth();

      console.log(numberOfSlidesToShift);
      for (var i = 0;  i < numberOfSlidesToShift; i++) {
        // move last <li> to front
        $(el).find('ul.slideshow li:first').before($(el).find('ul.slideshow li:last'));
        $(el).find('ul.slideshow').css('margin-left', '-=' + slideWidth);
      }

      $(el).find('ul.slideshow').animate({
        'margin-left': '+=' + slideWidth * numberOfSlidesToShift
      }, options.slideSpeed, options.slideTransitionEasing);


      if (options.auto) {
        this.start(el, options);
      }
    },

    setActiveDot: function(el, options, slideIndex) {
      slideIndex += 1;

      $(el).find('.slideshow-dot-nav li a').removeClass('fa-circle').addClass('fa-circle-o');
      $(el).find('.slideshow-dot-nav li:nth-child('+ slideIndex +') a').removeClass('fa-circle-o').addClass('fa-circle');
    }
  };

  // A really lightweight plugin wrapper around the constructor,
  // preventing against multiple instantiations
  $.fn[pluginName] = function ( options ) {
    return this.each(function () {
      if (!$.data(this, "plugin_" + pluginName)) {
        $.data(this, "plugin_" + pluginName,
          new Plugin( this, options ));
      }
    });
  };

})( jQuery, window, document );
